﻿using Microsoft.Extensions.Logging;
using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantView.Forms;
using MotorPlantContracts.DI;

namespace MotorPlantView
{
	public partial class FormEngines : Form
	{
		private readonly ILogger _logger;
		private readonly IEngineLogic _logic;
		public FormEngines(ILogger<FormEngines> logger, IEngineLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
		}

		private void FormEngines_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void LoadData()
		{
			try
			{
				dataGridView.FillAndConfigGrid(_logic.ReadList(null));
				_logger.LogInformation("Загрузка двигателей");

			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки двигателей");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonAdd_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormEngine>();
			if (form.ShowDialog() == DialogResult.OK)
			{
				LoadData();
			}
		}

		private void ButtonUpd_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				var form = DependencyManager.Instance.Resolve<FormEngine>();

				form.Id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				if (form.ShowDialog() == DialogResult.OK)
				{
					LoadData();
				}

			}
		}

		private void ButtonDel_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
					_logger.LogInformation("Удаление изделия");
					try
					{
						if (!_logic.Delete(new EngineBindingModel
						{
							Id = id
						}))
						{
							throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
						}
						LoadData();
					}
					catch (Exception ex)
					{
						_logger.LogError(ex, "Ошибка удаления изделия");
						MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
		}
		private void ButtonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}
	}
}
