﻿using Microsoft.Extensions.Logging;
using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.DI;
using System.Windows.Forms;
namespace MotorPlantView.Forms
{
	public partial class ImplementersForm : Form
	{
		private readonly ILogger _logger;
		private readonly IImplementerLogic _logic;

		public ImplementersForm(ILogger<ImplementersForm> logger, IImplementerLogic implementerLogic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = implementerLogic;
		}

		private void LoadData()
		{
			try
			{
				DataGridView.FillAndConfigGrid(_logic.ReadList(null));
				_logger.LogInformation("Загрузка исполнителей");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки исполнителей");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
				MessageBoxIcon.Error);
			}
		}

		private void FormImplementers_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void buttonAdd_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(ImplementerForm));
			if (service is ImplementerForm form)
			{
				if (form.ShowDialog() == DialogResult.OK)
				{
					LoadData();
				}
			}
		}

		private void buttonUpd_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				var service = Program.ServiceProvider?.GetService(typeof(ImplementerForm));
				if (service is ImplementerForm form)
				{
					form.Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
					if (form.ShowDialog() == DialogResult.OK)
					{
						LoadData();
					}
				}
			}
		}
	
		
		private void buttonDel_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				if (MessageBox.Show("Удалить запись?", "Вопрос",
				MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
					_logger.LogInformation("Удаление исполнителя");
					try
					{
						if (!_logic.Delete(new ImplementerBindingModel
						{
							Id = id
						}))
						{
							throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
						}
						LoadData();
					}
					catch (Exception ex)
					{
						_logger.LogError(ex, "Ошибка удаления исполнителя");
						MessageBox.Show(ex.Message, "Ошибка",
						MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
		}
		private void buttonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}
	}
}